//config.cpp



#include <ctype.h>
#include <gnome.h>
#include "list.h"
#include "config.h"


Config::Config(gchar filename[])
{
	gint n = strlen(g_get_home_dir()) + strlen(filename) + 2;
	_filename = (gchar*)g_malloc(sizeof(gchar) * n);
	snprintf(_filename, sizeof(gchar) * n, "%s/%s", g_get_home_dir(), filename);
}

Config::~Config()
{
	g_free(_filename);

	Section* s;
	Variable* v;
	while((s = (Section*)_sections.get(0)))
	{
		while((v = (Variable*)s->variables.get(0)))
		{
			s->variables.remove(v);
			g_free(v->name);
			g_free(v->value);
			delete v;
		}
		_sections.remove(s);
		g_free(s->name);
		delete s;
	}
}


//useful
gint Config::open(gboolean write)
{
	_f = fopen(_filename, write ? "w" : "r");
	if(_f == NULL)
		return FALSE;

	gchar section[SECTION_MAX_LENGTH+1];
	section[0] = '\0';
	gchar variable[VARIABLE_MAX_LENGTH+1];
	gchar value[VALUE_MAX_LENGTH+1];

	typedef enum _Situation
	{
		NONE,
		SECTION,
		VALUE,
		COMMENT
	} Situation;

	gboolean is_special = FALSE;

	Situation situation = NONE;
	gchar c;
	gint i = 0;

	while(fread(&c, sizeof(gchar), 1, _f))
	{
		switch(situation)
		{
			case NONE:
				if(i == 0 && c == '[')
					situation = SECTION;
				else if(i == 0 && c == '#')
					situation = COMMENT;
				else if(c == '=' && !is_special)
				{
					variable[i] = '\0';
					i = 0;
					situation = VALUE;
				}
				else if(c == '\n' && !is_special)
					i = 0;
				else
					variable[i++] = c;
				break;
			case SECTION:
				if(c == ']')
				{
					section[i] = '\0';
					i = 0;
					situation = NONE;
				}
				else
					section[i++] = c;
				break;
			case VALUE:
				if(c == '\n' && !is_special)
				{
					value[i] = '\0';
					i = 0;
					situation = NONE;
					setVariable(section, variable, value);
				}
				else
					value[i++] = c;
				break;
			case COMMENT:
				if(c == '\n' && !is_special)
					situation = NONE;
				break;
		};
	}

	return TRUE;
}

gint Config::close()
{
	if(_f == NULL)
	{
		g_print("Error: tried to close configuration file while not opened!");
		return FALSE;
	}

	//write all the stuff
	Section* s;
	gchar* str;
	for(gint i = 0; (s = (Section*)_sections.get(i)); i++)
	{
		str = "[";
		fwrite(str, strlen(str), 1, _f);
		fwrite(s->name, strlen(s->name), 1, _f);
		str = "]";
		fwrite(str, strlen(str), 1, _f);
		str = "\n";
		fwrite(str, strlen(str), 1, _f);

		Variable* v;
		for(gint i = 0; (v = (Variable*)s->variables.get(i)); i++)
		{
			fwrite(v->name, strlen(v->name), 1, _f);
			str = "=";
			fwrite(str, strlen(str), 1, _f);
			fwrite(v->value, strlen(v->value), 1, _f);
			str = "\n";
			fwrite(str, strlen(str), 1, _f);
		}

		str = "\n";
		fwrite(str, strlen(str), 1, _f);
	}

	if(fclose(_f) == 0)
		return TRUE;
	return FALSE;
}

//save
/*gint Config::addSection()
{
}*/

void Config::setVariable(gchar section[], gchar variable[], gchar value[])
{
	if(_f == NULL)
		return;

	Section* s;
	for(gint i = 0; (s = ((Section*)_sections.get(i))); i++)
	{
		if(strcmp(s->name, section) == 0)
		{
			Variable* v;
			for(gint i = 0; (v = ((Variable*)s->variables.get(i))); i++)
			{
				if(strcmp(variable, v->name) == 0)
					//we have to set an existing variable
				{
					g_free(v->value);
					v->value = g_strdup(value);
//					s->variables.append(v);
					return;
				}
			}

			//we have to create a new variable in existing section
			v = new Variable;
			v->name = g_strdup(variable);
			v->value = g_strdup(value);
			s->variables.append(v);
			return;
		}
	}

	//we have to create a new section
	s = new Section;
	s->name = g_strdup(section);
	Variable* v = new Variable;
	v->name = g_strdup(variable);
	v->value = g_strdup(value);
	s->variables.append(v);
	_sections.append(s);
}

void Config::getVariable(gchar section[], gchar variable[], gchar* value[])
{
	if(_f == NULL)
		return;

	Section* s;
	for(gint i = 0; (s = ((Section*)_sections.get(i))); i++)
	{
		if(strcmp(s->name, section) == 0)
		{
			Variable* v;
			for(gint i = 0; (v = ((Variable*)s->variables.get(i))); i++)
			{
				if(strcmp(variable, v->name) == 0)
				{
					*value = g_strdup(v->value);
					return;
				}
			}
			*value = NULL;
			return;
		}
	}

	//we didn't find anything
	*value = NULL;
}

void Config::deleteSection(gchar section[])
{
	if(_f == NULL)
		return;

	Section* s;
	for(gint i = 0; (s = ((Section*)_sections.get(i))); i++)
	{
		if(strcmp(s->name, section) == 0)
		{
			Variable* v;
			for(; (v = ((Variable*)s->variables.get(0)));)
			{
				g_free(v->name);
				g_free(v->value);
				s->variables.remove(v);
				delete v;
			}
			_sections.remove(s);
			g_free(s->name);
			delete s;
			return;
		}
	}
}

void Config::deleteVariable(gchar section[], gchar variable[])
{
	if(_f == NULL)
		return;

	Section* s;
	for(gint i = 0; (s = ((Section*)_sections.get(i))); i++)
	{
		if(strcmp(s->name, section) == 0)
		{
			Variable* v;
			for(gint i = 0; (v = ((Variable*)s->variables.get(i))); i++)
			{
				if(strcmp(variable, v->name) == 0)
				{
					s->variables.remove(v);
					delete v;
					return;
				}
			}
		}
	}
}
